﻿Public Class EventPackList : Inherits List(Of EventPack)

	Public Sub AddEventHandlerTo(EventRaiser As Object, EventReceiver As Object, EventName As String, EventHandlerName As String)
		If Any(Function(x) x.Control Is EventRaiser AndAlso x.EventName = EventName) Then Return

		Add(New EventPack(EventRaiser, EventName, EventHandlerName, Nothing))
		For Each EventPack In Me
			Dim TargetEventHandlerMethod = EventReceiver.GetType.GetMethod(EventPack.EventHandlerName)
			Dim EventInfo = EventRaiser.GetType.GetEvent(EventPack.EventName)
			Dim EventHandler = [Delegate].CreateDelegate(EventInfo.EventHandlerType, EventReceiver, EventPack.EventHandlerName)
			EventPack.EventHandler = EventHandler
		Next
	End Sub

	Public Sub ActivateAllEventHandlers()
		Me.Where(Function(x) Not x.IsActive).ForEach(Sub(x) ActivateEventHandler(x))
	End Sub

	Public Sub DeactivateAllEventHandlers()
		Me.Where(Function(x) x.IsActive).ForEach(Sub(EventPack) DeactivateEventHandler(EventPack))
	End Sub

	Private Sub ActivateEventHandler(EventPack As EventPack)
		Dim EventInfo = EventPack.Control.GetType.GetEvent(EventPack.EventName)
		EventInfo.AddEventHandler(EventPack.Control, EventPack.EventHandler)
		EventPack.IsActive = True
	End Sub

	Private Sub DeactivateEventHandler(EventPack As EventPack)
		Dim EventInfo = EventPack.Control.GetType.GetEvent(EventPack.EventName)
		EventInfo.RemoveEventHandler(EventPack.Control, EventPack.EventHandler)
		EventPack.IsActive = False
	End Sub

End Class