﻿Imports System.Runtime.CompilerServices

Friend Module Extensions

	Public ReadOnly Property NullString As String = Nothing

	<Extension> Public Function GetRows(Dgv As DataGridView) As IEnumerable(Of DataGridViewRow)
		Return Dgv.Rows.Cast(Of DataGridViewRow)
	End Function

	<Extension> Public Function LastRowIndex(Dgv As DataGridView) As Integer
		If Dgv.Rows.Count = 0 Then Return -1

		Dim LastRowIndexOffset = If(Dgv.AllowUserToAddRows, 2, 1)
		Return Dgv.Rows.Count - LastRowIndexOffset
	End Function

	<Extension> Public Sub ClearContent(Dgv As DataGridView)
		Dgv.Rows.Clear()
	End Sub

	<Extension> Public Function Text(Cell As DataGridViewCell) As String
		If Cell.NotExists OrElse Cell.Value Is Nothing Then Return Nothing

		Return Cell.Value.ToString
	End Function

	<Extension> Public Function GetAllNodes(TreeView As TreeView) As IEnumerable(Of TreeNode)
		Return GetAllTreeViewNodes(TreeView.Nodes.Cast(Of TreeNode))
	End Function

	Private Function GetAllTreeViewNodes(Nodes As IEnumerable(Of TreeNode)) As IEnumerable(Of TreeNode)
		Dim FinalList As New List(Of TreeNode)
		For Each Node In Nodes
			FinalList.Add(Node)
			FinalList.AddRange(GetAllTreeViewNodes(Node.Nodes.Cast(Of TreeNode)))
		Next
		Return FinalList
	End Function

	<Extension()> Public Function ParseToInteger(Text As String) As Integer
		Dim Result = 0
		If Not Integer.TryParse(Text, Result) Then Return 0

		Return Result
	End Function

	<Extension> Public Function IsNull(Pointer As IntPtr) As Boolean
		Return Pointer = IntPtr.Zero
	End Function

	<Extension> Public Function Exists([Object] As Object) As Boolean
		Return [Object] IsNot Nothing
	End Function

	<Extension> Public Function NotExists([Object] As Object) As Boolean
		Return [Object] Is Nothing
	End Function

	<Extension> Public Sub ForEach(Of T)(IEnumerable As IEnumerable(Of T), Action As Action(Of T))
		For Each Element In IEnumerable
			Action(Element)
		Next
	End Sub

	<Extension> Public Function None(Of T)(List As IEnumerable(Of T), Predicate As Func(Of T, Boolean)) As Boolean
		Return Not List.Any(Function(x) Predicate(x))
	End Function

	<Extension> Public Function IsBetween(Of T As IComparable)(Thing As T, LowerBoundary As T, UpperBoundary As T) As Boolean
		Return Thing.CompareTo(LowerBoundary) >= 0 AndAlso Thing.CompareTo(UpperBoundary) <= 0
	End Function

	<Extension> Public Sub ToggleBetween(Of T)(ByRef TargetObject As T, Value1 As T, Value2 As T)
		TargetObject = If(TargetObject.Equals(Value1), Value2, Value1)
	End Sub

	<Extension> Public Function ExistsAsFile(FilePath As String) As Boolean
		Return IO.File.Exists(FilePath)
	End Function

	<Extension()> Public Function IsEmpty(TestString As String) As Boolean
		Return TestString.NotExists OrElse String.IsNullOrEmpty(TestString)
	End Function

End Module