﻿Imports System.Runtime.InteropServices
Imports Interop.UIAutomationClient

Public Class MenuItemList

	<DllImport("user32.dll")> Private Shared Function GetMenu(hWnd As IntPtr) As IntPtr : End Function
	<DllImport("user32.dll")> Private Shared Function GetMenuItemCount(hMenu As IntPtr) As Int32 : End Function
	<DllImport("user32.dll")> Private Shared Function GetMenuItemID(hMenu As IntPtr, nPos As Integer) As Integer : End Function
	<DllImport("user32.dll", CharSet:=CharSet.Unicode)> Private Shared Function GetMenuString(hMenu As IntPtr, wIDItem As Integer, lpString As String, nMaxCount As Integer, wFlag As Integer) As Integer : End Function
	<DllImport("user32.dll")> Private Shared Function GetSubMenu(hMenu As IntPtr, nPos As Integer) As IntPtr : End Function

	Friend ReadOnly Property MenuItems As IEnumerable(Of MenuItemData)
		Get
			Return Items
		End Get
	End Property

	Private ReadOnly Items As New List(Of MenuItemData)

	Friend Sub RetrieveMenuItemsFor(Box As IUIAutomationElement)
		ClearItemList()
		CollectItemsForExistingMenu(GetMenu(Box.CurrentNativeWindowHandle))
	End Sub

	Private Sub ClearItemList()
		Items.Clear()
	End Sub

	Private Sub CollectItemsForExistingMenu(MenuHandle As IntPtr)
		If MenuHandle.IsNull Then Return

		CollectItemsFor(MenuHandle, GetMenuItemCount(MenuHandle))
	End Sub

	Private Sub CollectItemsFor(MenuHandle As IntPtr, ItemCount As Integer)
		If ItemCount <= 0 Then Return

		Enumerable.Range(0, ItemCount).ForEach(Sub(x) CollectItemData(MenuHandle, x))
	End Sub

	Private Sub CollectItemData(MenuHandle As IntPtr, ItemIndex As Integer)
		RememberItemData(MenuHandle, ItemIndex)
		CollectItemsForExistingMenu(GetSubMenu(MenuHandle, ItemIndex))
	End Sub

	Private Sub RememberItemData(MenuHandle As IntPtr, Index As Integer)
		Dim Title = GetTitleFor(MenuHandle, Index)
		Dim ItemID = GetMenuItemID(MenuHandle, Index)
		Dim SubMenuHandle = GetSubMenu(MenuHandle, Index)
		Dim MenuName = Items.FirstOrDefault(Function(x) x.SubMenuHandle = MenuHandle)?.Title
		If MenuName.IsEmpty AndAlso ItemID = -1 Then MenuName = "(Hauptmenü)"
		Items.Add(New MenuItemData With {.Title = Title, .ItemID = ItemID, .MenuHandle = MenuHandle, .SubMenuHandle = SubMenuHandle, .MenuName = MenuName})
	End Sub

	Private Function GetTitleFor(MenuHandle As IntPtr, Index As Integer) As String
		Dim MenuTitle = New String(" "c, 128)
		Dim MenuNameLength = GetMenuString(MenuHandle, Index, MenuTitle, 128, 1024)
		GetMenuString(MenuHandle, Index, MenuTitle, MenuNameLength + 1, 1024)
		Return MenuTitle
	End Function

End Class