Partial Class FrmMain

	Private Function GetSelectedAutomationID() As String
		Return DgvControlProperties.GetRows.First(Function(x) x.Cells(0).Text = "CurrentAutomationId").Cells(1).Text
	End Function

	Private Function GetPropertyDataFrom(SimpleProperty As Reflection.PropertyInfo) As (Name As String, Value As String)
		Dim Value As Object = Nothing
		Try
			Value = SimpleProperty.GetValue(BsDescendantAutomationElements.Current)
		Catch Ex As Exception When TypeOf Ex Is Reflection.TargetInvocationException OrElse TypeOf Ex Is Reflection.TargetException
		End Try

		Return (SimpleProperty.Name, Value?.ToString)
	End Function

	Private Function GetComplexPropertyDataFrom(ComplexProperty As Reflection.PropertyInfo) As IEnumerable(Of (Name As String, Value As String))
		If ComplexProperty.PropertyType.IsPrimitive Then Return New List(Of (Name As String, Value As String))

		Dim Value As Object = Nothing
		Dim PropertyData As New List(Of (Name As String, Value As String))
		Try
			For Each Field In ComplexProperty.PropertyType.GetFields.Where(Function(x) x.Name <> "Empty")
				Dim PropertyValue = ComplexProperty.GetValue(BsDescendantAutomationElements.Current)
				Value = Field.GetValue(PropertyValue)
				PropertyData.Add(("--> " & Field.Name, Value?.ToString))
			Next
		Catch Ex As Exception When TypeOf Ex Is Reflection.TargetInvocationException OrElse TypeOf Ex Is Reflection.TargetException
		End Try
		Return PropertyData
	End Function

	Private Function GetAllAutomationElementProperties() As IEnumerable(Of Reflection.PropertyInfo)
		Return GetType(Interop.UIAutomationClient.IUIAutomationElement).GetProperties
	End Function

	Private Function GetPropertyDataFor(SingleProperty As Reflection.PropertyInfo) As IEnumerable(Of (Name As String, Value As String))
		Dim PropertyData As New List(Of (Name As String, Value As String)) From {GetPropertyDataFrom(SingleProperty)}
		PropertyData.AddRange(GetComplexPropertyDataFrom(SingleProperty))
		Return PropertyData
	End Function

	Private Function GetMethodDataPack() As MethodDataPack
		Return New MethodDataPack With {
			.PatternName = BsPatterns.Current.Name,
			.PatternID = BsPatterns.Current.PatternID,
			.MethodName = BsPatternMethodInfos.Current.Name,
			.ParameterValues = GetParameterValues()
		}
	End Function

	Private Function GetParameterValues() As IEnumerable(Of Object)
		Dim ParameterValues As New List(Of Object)
		For i = 0 To DgvParameterList.LastRowIndex
			Dim CurrentParameterValue = DgvParameterList.Item(2, i).Value
			Dim TargetType = DgvParameterList.Item(1, i).Value.ToString
			Try
				ParameterValues.Add(Convert.ChangeType(CurrentParameterValue, Type.GetType(TargetType)))
			Catch ICEx As InvalidCastException
			End Try
		Next
		Return ParameterValues
	End Function

	Private Function GetPropertyDataPack() As PropertyDataPack
		Return New PropertyDataPack With {
			.PatternName = BsPatterns.Current.Name,
			.PatternID = BsPatterns.Current.PatternID,
			.PropertyName = CbxPatternProperties.SelectedItem.ToString
		}
	End Function

	Private Function CurrentMainBoxIsDesktop() As Boolean
		If BsAutomationElements.HasNoCurrent Then Return True

		Return BsAutomationElements.Current.CurrentProcessId = UIWrapper.GetRootElement.CurrentProcessId AndAlso BsAutomationElements.Current.CurrentName = UIWrapper.GetRootElement.CurrentName
	End Function

	Private Function GetParameterListForCodeRecorder() As String
		Dim ParameterList As New List(Of String)
		For Each ParameterRow In DgvParameterList.GetRows
			If ParameterRow.Cells(1).Text <> "System.String" Then
				ParameterList.Add(ParameterRow.Cells(2).Value.ToString)
			Else
				ParameterList.Add($"""{ParameterRow.Cells(2).Value}""")
			End If
		Next
		Return String.Join(", ", ParameterList)
	End Function

	Private Function NoMenuItemIsSelected() As Boolean
		Return DgvAppMenu.SelectedRows.Count = 0
	End Function

	Private Function GetSelectedMenuItemID() As Integer
		Return DgvAppMenu.SelectedRows(0).Cells(2).Text.ParseToInteger
	End Function

	Private Function GetLastNodeFor(CurrentNode As TreeNode) As TreeNode
		If CurrentNode.NotExists Then Return TvAppStructure.Nodes.Cast(Of TreeNode).Last

		Return CurrentNode.LastNode
	End Function

	Private Function FilePathExists(FilePath As String) As Boolean
		Return FilePath.ExistsAsFile
	End Function

	Private Function GetSubBoxBorder() As Rectangle
		Dim RectangleRow = DgvControlProperties.GetRows.SingleOrDefault(Function(x) x.Cells(0).Text = "CurrentBoundingRectangle")
		If RectangleRow.NotExists Then Return Nothing

		Return New Rectangle With {
			.Left = DgvControlProperties.Item(1, RectangleRow.Index + 1).Text.ParseToInteger,
			.Top = DgvControlProperties.Item(1, RectangleRow.Index + 2).Text.ParseToInteger,
			.Right = DgvControlProperties.Item(1, RectangleRow.Index + 3).Text.ParseToInteger,
			.Bottom = DgvControlProperties.Item(1, RectangleRow.Index + 4).Text.ParseToInteger
		}
	End Function

End Class